@@ -30,6 +30,7 @@
 
 #include "glib.h"
 #include "gprintfint.h"
+#include "gthreadinit.h"
 
 #ifdef G_PLATFORM_WIN32
 #define STRICT
@@ -96,7 +97,7 @@ extern const char **_g_charset_get_aliases (const char *canonical_name);
  * @to_codeset: destination codeset
  * @from_codeset: source codeset
  * 
- * Same as the standard UNIX routine <function>iconv_open()</function>, but
+ * Same as the standard UNIX routine iconv_open(), but
  * may be implemented via libiconv on UNIX flavors that lack
  * a native implementation.
  * 
@@ -148,7 +149,7 @@ g_iconv_open (const gchar  *to_codeset,
  * @outbuf: converted output bytes
  * @outbytes_left: inout parameter, bytes available to fill in @outbuf
  * 
- * Same as the standard UNIX routine <function>iconv()</function>, but
+ * Same as the standard UNIX routine iconv(), but
  * may be implemented via libiconv on UNIX flavors that lack
  * a native implementation.
  *
@@ -173,7 +174,7 @@ g_iconv (GIConv   converter,
  * g_iconv_close:
  * @converter: a conversion descriptor from g_iconv_open()
  *
- * Same as the standard UNIX routine <function>iconv_close()</function>, but
+ * Same as the standard UNIX routine iconv_close(), but
  * may be implemented via libiconv on UNIX flavors that lack
  * a native implementation. Should be called to clean up
  * the conversion descriptor from g_iconv_open() when
@@ -235,7 +236,7 @@ iconv_cache_init (void)
  *
  * Returns a pointer to the newly allocated cache bucket.
  **/
-struct _iconv_cache_bucket *
+static struct _iconv_cache_bucket *
 iconv_cache_bucket_new (const gchar *key, GIConv cd)
 {
   struct _iconv_cache_bucket *bucket;
@@ -651,7 +652,7 @@ g_convert_with_iconv (const gchar *str,
  *                present in the target encoding. (This must be
  *                in the target encoding), if %NULL, characters
  *                not in the target encoding will be represented
- *                as Unicode escapes \x{XXXX} or \x{XXXXXX}.
+ *                as Unicode escapes \uxxxx or \Uxxxxyyyy.
  * @bytes_read:   location to store the number of bytes in the
  *                input string that were successfully converted, or %NULL.
  *                Even if the conversion was successful, this may be 
@@ -667,7 +668,7 @@ g_convert_with_iconv (const gchar *str,
  * in the output. Note that it is not guaranteed that the specification
  * for the fallback sequences in @fallback will be honored. Some
  * systems may do a approximate conversion from @from_codeset
- * to @to_codeset in their <function>iconv()</function> functions, 
+ * to @to_codeset in their iconv() functions, 
  * in which case GLib will simply return that approximate conversion.
  *
  * Return value: If the conversion was successful, a newly allocated
@@ -806,8 +807,7 @@ g_convert_with_fallback (const gchar *str,
 		  if (!fallback)
 		    { 
 		      gunichar ch = g_utf8_get_char (p);
-		      insert_str = g_strdup_printf ("\\x{%0*X}",
-						    (ch < 0x10000) ? 4 : 6,
+		      insert_str = g_strdup_printf (ch < 0x10000 ? "\\u%04x" : "\\U%08x",
 						    ch);
 		    }
 		  else
@@ -948,7 +948,6 @@ g_locale_to_utf8 (const gchar  *opsysstring,
   else
     return g_convert (opsysstring, len, 
 		      "UTF-8", charset, bytes_read, bytes_written, error);
-
 }
 
 /**
@@ -992,32 +991,114 @@ g_locale_from_utf8 (const gchar *utf8string,
 }
 
 #ifndef G_PLATFORM_WIN32
-static gboolean
-have_broken_filenames (void)
-{
-  static gboolean initialized = FALSE;
-  static gboolean broken;
-  
-  if (initialized)
-    return broken;
 
-  broken = (getenv ("G_BROKEN_FILENAMES") != NULL);
-  
-  initialized = TRUE;
-  
-  return broken;
+typedef struct _GFilenameCharsetCache GFilenameCharsetCache;
+
+struct _GFilenameCharsetCache {
+  gboolean is_utf8;
+  gchar *charset;
+  gchar *filename_charset;
+};
+
+static void
+filename_charset_cache_free (gpointer data)
+{
+  GFilenameCharsetCache *cache = data;
+  g_free (cache->charset);
+  g_free (cache->filename_charset);
+  g_free (cache);
 }
-#endif /* !G_PLATFORM_WIN32 */
+
+/*
+ * get_filename_charset:
+ * @charset: return location for the name of the filename encoding 
+ *
+ * Determines the character set used for filenames by consulting the 
+ * environment variables G_FILENAME_ENCODING and G_BROKEN_FILENAMES. 
+ *
+ * G_FILENAME_ENCODING may be set to a comma-separated list of character 
+ * set names. The special token "@locale" is taken to mean the character set 
+ * for the current locale. The first character set from the list is taken 
+ * as the filename encoding. 
+ * If G_FILENAME_ENCODING is not set, but G_BROKEN_FILENAMES is, the
+ * character set of the current locale is taken as the filename encoding.
+ *
+ * The returned @charset belongs to GLib and must not be freed.
+ * 
+ * Return value: %TRUE if the charset used for filename is UTF-8.
+ */
+static gboolean
+get_filename_charset (const gchar **filename_charset)
+{
+  static GStaticPrivate cache_private = G_STATIC_PRIVATE_INIT;
+  GFilenameCharsetCache *cache = g_static_private_get (&cache_private);
+  const gchar *charset;
+  
+  if (!cache)
+    {
+      cache = g_new0 (GFilenameCharsetCache, 1);
+      g_static_private_set (&cache_private, cache, filename_charset_cache_free);
+    }
+
+  g_get_charset (&charset);
+
+  if (!(cache->charset && strcmp (cache->charset, charset) == 0))
+    {
+      const gchar *new_charset;
+      gchar *p, *q;
+
+      g_free (cache->charset);
+      g_free (cache->filename_charset);
+      cache->charset = g_strdup (charset);
+      
+      p = getenv ("G_FILENAME_ENCODING");
+      if (p != NULL) 
+	{
+	  q = strchr (p, ',');
+	  if (!q) 
+	    q = p + strlen (p);
+
+	  if (strncmp ("@locale", p, q - p) == 0)
+	    {
+	      cache->is_utf8 = g_get_charset (&new_charset);
+	      cache->filename_charset = g_strdup (new_charset);
+	    }
+	  else
+	    {
+	      cache->filename_charset = g_strndup (p, q - p);
+	      cache->is_utf8 = (strcmp (cache->filename_charset, "UTF-8") == 0);
+	    }
+	}
+      else if (getenv ("G_BROKEN_FILENAMES") != NULL)
+	{
+	  cache->is_utf8 = g_get_charset (&new_charset);
+	  cache->filename_charset = g_strdup (new_charset);
+	}
+      else 
+	{
+	  cache->filename_charset = g_strdup ("UTF-8");
+	  cache->is_utf8 = TRUE;
+	}
+    }
+
+  if (filename_charset)
+    *filename_charset = cache->filename_charset;
+
+  return cache->is_utf8;
+}
+
+#else /* G_PLATFORM_WIN32 */
+#define get_filename_charset (charset) TRUE
+#endif /* G_PLATFORM_WIN32 */
 
 /* This is called from g_thread_init(). It's used to
  * initialize some static data in a threadsafe way.
  */
 void 
-g_convert_init (void)
+_g_convert_thread_init (void)
 {
-#ifndef G_PLATFORM_WIN32
-  (void)have_broken_filenames ();
-#endif /* !G_PLATFORM_WIN32 */
+  const gchar *dummy;
+  (void) get_filename_charset (&dummy);
 }
 
 /**
@@ -1050,19 +1131,13 @@ g_filename_to_utf8 (const gchar *opsysstring,
 		    gsize       *bytes_written,
 		    GError     **error)
 {
-#ifdef G_PLATFORM_WIN32
-  return g_locale_to_utf8 (opsysstring, len,
-			   bytes_read, bytes_written,
-			   error);
-#else  /* !G_PLATFORM_WIN32 */
-      
-  if (have_broken_filenames ())
-    return g_locale_to_utf8 (opsysstring, len,
-			     bytes_read, bytes_written,
-			     error);
-  else
+  const gchar *charset;
+
+  if (get_filename_charset (&charset))
     return strdup_len (opsysstring, len, bytes_read, bytes_written, error);
-#endif /* !G_PLATFORM_WIN32 */
+  else
+    return g_convert (opsysstring, len, 
+		      "UTF-8", charset, bytes_read, bytes_written, error);
 }
 
 /**
@@ -1094,18 +1169,13 @@ g_filename_from_utf8 (const gchar *utf8string,
 		      gsize       *bytes_written,
 		      GError     **error)
 {
-#ifdef G_PLATFORM_WIN32
-  return g_locale_from_utf8 (utf8string, len,
-			     bytes_read, bytes_written,
-			     error);
-#else  /* !G_PLATFORM_WIN32 */
-  if (have_broken_filenames ())
-    return g_locale_from_utf8 (utf8string, len,
-			       bytes_read, bytes_written,
-			       error);
-  else
+  const gchar *charset;
+
+  if (get_filename_charset (&charset))
     return strdup_len (utf8string, len, bytes_read, bytes_written, error);
-#endif /* !G_PLATFORM_WIN32 */
+  else
+    return g_convert (utf8string, len,
+		      charset, "UTF-8", bytes_read, bytes_written, error);
 }
 
 /* Test of haystack has the needle prefix, comparing case
@@ -1399,9 +1469,9 @@ hostname_validate (const char *hostname)
  *               filename, or %NULL on an error.
  **/
 gchar *
-g_filename_from_uri (const char *uri,
-		     char      **hostname,
-		     GError    **error)
+g_filename_from_uri (const gchar *uri,
+		     gchar      **hostname,
+		     GError     **error)
 {
   const char *path_part;
   const char *host_part;
@@ -1533,9 +1603,9 @@ g_filename_from_uri (const char *uri,
  *               URI, or %NULL on an error.
  **/
 gchar *
-g_filename_to_uri   (const char *filename,
-		     const char *hostname,
-		     GError    **error)
+g_filename_to_uri   (const gchar *filename,
+		     const gchar *hostname,
+		     GError     **error)
 {
   char *escaped_uri;
   char *utf8_filename;
