@@ -645,22 +645,22 @@ g_file_get_contents (const gchar *filename,
  * g_mkstemp:
  * @tmpl: template filename
  *
- * Opens a temporary file. See the <function>mkstemp()</function> documentation
+ * Opens a temporary file. See the mkstemp() documentation
  * on most UNIX-like systems. This is a portability wrapper, which simply calls 
- * <function>mkstemp()</function> on systems that have it, and implements 
+ * mkstemp() on systems that have it, and implements 
  * it in GLib otherwise.
  *
  * The parameter is a string that should match the rules for
- * <function>mkstemp()</function>, i.e. end in "XXXXXX". The X string will 
+ * mkstemp(), i.e. end in "XXXXXX". The X string will 
  * be modified to form the name of a file that didn't exist.
  *
- * Return value: A file handle (as from <function>open()</function>) to the file
+ * Return value: A file handle (as from open()) to the file
  * opened for reading and writing. The file is opened in binary mode
  * on platforms where there is a difference. The file handle should be
- * closed with <function>close()</function>. In case of errors, -1 is returned.
+ * closed with close(). In case of errors, -1 is returned.
  */
-int
-g_mkstemp (char *tmpl)
+gint
+g_mkstemp (gchar *tmpl)
 {
 #ifdef HAVE_MKSTEMP
   return mkstemp (tmpl);
@@ -729,27 +729,27 @@ g_mkstemp (char *tmpl)
  * files (as returned by g_get_tmp_dir()). 
  *
  * @tmpl should be a string ending with six 'X' characters, as the
- * parameter to g_mkstemp() (or <function>mkstemp()</function>). 
+ * parameter to g_mkstemp() (or mkstemp()). 
  * However, unlike these functions, the template should only be a 
  * basename, no directory components are allowed. If template is %NULL, 
  * a default template is used.
  *
- * Note that in contrast to g_mkstemp() (and <function>mkstemp()</function>) 
+ * Note that in contrast to g_mkstemp() (and mkstemp()) 
  * @tmpl is not modified, and might thus be a read-only literal string.
  *
  * The actual name used is returned in @name_used if non-%NULL. This
  * string should be freed with g_free() when not needed any longer.
  *
- * Return value: A file handle (as from <function>open()</function>) to 
+ * Return value: A file handle (as from open()) to 
  * the file opened for reading and writing. The file is opened in binary 
  * mode on platforms where there is a difference. The file handle should be
- * closed with <function>close()</function>. In case of errors, -1 is returned 
+ * closed with close(). In case of errors, -1 is returned 
  * and @error will be set.
  **/
-int
-g_file_open_tmp (const char *tmpl,
-		 char      **name_used,
-		 GError    **error)
+gint
+g_file_open_tmp (const gchar *tmpl,
+		 gchar      **name_used,
+		 GError     **error)
 {
   int retval;
   const char *tmpdir;
@@ -1112,3 +1112,62 @@ g_build_filename (const gchar *first_element,
     }
 #endif
 }
+
+/**
+ * g_file_read_link:
+ * @filename: the symbolic link
+ * @error: return location for a #GError
+ *
+ * Reads the contents of the symbolic link @filename like the POSIX readlink() function.
+ * The returned string is in the encoding used for filenames. Use g_filename_to_utf8() to 
+ * convert it to UTF-8.
+ *
+ * Returns: A newly allocated string with the contents of the symbolic link, 
+ *          or %NULL if an error occurred.
+ *
+ * Since: 2.4
+ */
+gchar *
+g_file_read_link (const gchar *filename,
+	          GError     **error)
+{
+#ifdef HAVE_READLINK
+  gchar *buffer;
+  guint size;
+  gint read_size;    
+  
+  size = 256; 
+  buffer = g_malloc (size);
+  
+  while (TRUE) 
+    {
+      read_size = readlink (filename, buffer, size);
+      if (read_size < 0) {
+	g_free (buffer);
+	g_set_error (error,
+		     G_FILE_ERROR,
+		     g_file_error_from_errno (errno),
+		     _("Failed to read the symbolic link '%s': %s"),
+		     filename, g_strerror (errno));
+	
+	return NULL;
+      }
+    
+      if (read_size < size) 
+	{
+	  buffer[read_size] = 0;
+	  return buffer;
+	}
+      
+      size *= 2;
+      buffer = g_realloc (buffer, size);
+    }
+#else
+  g_set_error (error,
+	       G_FILE_ERROR,
+	       G_FILE_ERROR_INVAL,
+	       _("Symbolic links not supported"));
+	
+  return NULL;
+#endif
+}
