@@ -116,12 +116,20 @@ gchar *               g_strrstr_len    (const gchar  *haystack,
 					gssize        haystack_len,
 					const gchar  *needle);
 
+gboolean              g_str_has_suffix (const gchar  *str,
+					const gchar  *suffix);
+gboolean              g_str_has_prefix (const gchar  *str,
+					const gchar  *prefix);
+
 /* String to/from double conversion functions */
 
 gdouble	              g_strtod         (const gchar  *nptr,
 					gchar	    **endptr);
 gdouble	              g_ascii_strtod   (const gchar  *nptr,
 					gchar	    **endptr);
+guint64		      g_ascii_strtoull (const gchar *nptr,
+					gchar      **endptr,
+					guint        base);
 /* 29 bytes should enough for all possible values that
  * g_ascii_dtostr can produce.
  * Then add 10 for good measure */
@@ -223,6 +231,9 @@ gchar**               g_strdupv        (gchar       **str_array);
 gchar*                g_stpcpy         (gchar        *dest,
                                         const char   *src);
 
+G_CONST_RETURN gchar *g_strip_context  (const gchar *msgid, 
+					const gchar *msgval);
+
 G_END_DECLS
 
 #endif /* __G_STRFUNCS_H__ */
