@@ -173,7 +173,7 @@ my_strchrnul (const gchar *str, gchar c)
 
 #ifdef G_OS_WIN32
 
-gchar *inner_find_program_in_path (const gchar *program);
+static gchar *inner_find_program_in_path (const gchar *program);
 
 gchar*
 g_find_program_in_path (const gchar *program)
@@ -240,6 +240,9 @@ g_find_program_in_path (const gchar *program)
  *
  * Return value: absolute path, or NULL
  **/
+#ifdef G_OS_WIN32
+static
+#endif
 gchar*
 g_find_program_in_path (const gchar *program)
 {
@@ -641,6 +644,16 @@ g_get_current_dir (void)
   return dir;
 }
 
+/**
+ * g_getenv:
+ * @variable: the environment variable to get.
+ * 
+ * Returns an environment variable.
+ * 
+ * Return value: the value of the environment variable, or %NULL if the environment
+ * variable is not found. The returned string may be overwritten by the next call to g_getenv(),
+ * g_setenv() or g_unsetenv().
+ **/
 G_CONST_RETURN gchar*
 g_getenv (const gchar *variable)
 {
@@ -649,74 +662,145 @@ g_getenv (const gchar *variable)
 
   return getenv (variable);
 #else
-  G_LOCK_DEFINE_STATIC (getenv);
-  struct env_struct
-  {
-    gchar *key;
-    gchar *value;
-  } *env;
-  static GArray *environs = NULL;
+  GQuark quark;
   gchar *system_env;
-  guint length, i;
+  gchar *expanded_env;
+  guint length;
   gchar dummy[2];
 
   g_return_val_if_fail (variable != NULL, NULL);
   
-  G_LOCK (getenv);
-
-  if (!environs)
-    environs = g_array_new (FALSE, FALSE, sizeof (struct env_struct));
-
-  /* First we try to find the envinronment variable inside the already
-   * found ones.
-   */
-
-  for (i = 0; i < environs->len; i++)
-    {
-      env = &g_array_index (environs, struct env_struct, i);
-      if (strcmp (env->key, variable) == 0)
-	{
-	  g_assert (env->value);
-	  G_UNLOCK (getenv);
-	  return env->value;
-	}
-    }
-
-  /* If not found, we ask the system */
-
   system_env = getenv (variable);
   if (!system_env)
-    {
-      G_UNLOCK (getenv);
-      return NULL;
-    }
+    return NULL;
 
-  /* On Windows NT, it is relatively typical that environment variables
-   * contain references to other environment variables. Handle that by
-   * calling ExpandEnvironmentStrings.
+  /* On Windows NT, it is relatively typical that environment
+   * variables contain references to other environment variables. If
+   * so, use ExpandEnvironmentStrings(). (If all software was written
+   * in the best possible way, such environment variables would be
+   * stored in the Registry as REG_EXPAND_SZ type values, and would
+   * then get automatically expanded before the program sees them. But
+   * there is broken software that stores environment variables as
+   * REG_SZ values even if they contain references to other
+   * environment variables.
    */
 
-  g_array_set_size (environs, environs->len + 1);
-
-  env = &g_array_index (environs, struct env_struct, environs->len - 1);
+  if (strchr (system_env, '%') == NULL)
+    {
+      /* No reference to other variable(s), return value as such. */
+      return system_env;
+    }
 
   /* First check how much space we need */
   length = ExpandEnvironmentStrings (system_env, dummy, 2);
-
-  /* Then allocate that much, and actualy do the expansion and insert
-   * the new found pair into our buffer 
-   */
-
-  env->value = g_malloc (length);
-  env->key = g_strdup (variable);
-
-  ExpandEnvironmentStrings (system_env, env->value, length);
-
-  G_UNLOCK (getenv);
-  return env->value;
+  
+  expanded_env = g_malloc (length);
+  
+  ExpandEnvironmentStrings (system_env, expanded_env, length);
+  
+  quark = g_quark_from_string (expanded_env);
+  g_free (expanded_env);
+  
+  return g_quark_to_string (quark);
 #endif
 }
 
+/**
+ * g_setenv:
+ * @variable: the environment variable to set, must not contain '='.
+ * @value: the value for to set the variable to.
+ * @overwrite: whether to change the variable if it already exists.
+ *
+ * Sets an environment variable.
+ *
+ * Note that on some systems, the memory used for the variable and its value 
+ * can't be reclaimed later.
+ *
+ * Returns: %FALSE if the environment variable couldn't be set.
+ *
+ * Since: 2.4
+ */
+gboolean
+g_setenv (const gchar *variable, 
+	  const gchar *value, 
+	  gboolean     overwrite)
+{
+  gint result;
+#ifndef HAVE_SETENV
+  gchar *string;
+#endif
+
+  g_return_val_if_fail (strchr (variable, '=') == NULL, FALSE);
+
+#ifdef HAVE_SETENV
+  result = setenv (variable, value, overwrite);
+#else
+  if (!overwrite && g_getenv (variable) != NULL)
+    return TRUE;
+  
+  /* This results in a leak when you overwrite existing
+   * settings. It would be fairly easy to fix this by keeping
+   * our own parallel array or hash table.
+   */
+  string = g_strconcat (variable, "=", value, NULL);
+  result = putenv (string);
+#endif
+  return result == 0;
+}
+
+#ifndef HAVE_UNSETENV     
+/* According to the Single Unix Specification, environ is not in 
+ * any system header, although unistd.h often declares it.
+ */
+extern char **environ;
+#endif
+           
+/**
+ * g_unsetenv:
+ * @variable: the environment variable to remove, must not contain '='.
+ * 
+ * Removes an environment variable from the environment.
+ *
+ * Note that on some systems, the memory used for the variable and its value 
+ * can't be reclaimed. Furthermore, this function can't be guaranteed to operate in a 
+ * threadsafe way.
+ *
+ * Since: 2.4 
+ **/
+void
+g_unsetenv (const gchar *variable)
+{
+#ifdef HAVE_UNSETENV
+  g_return_if_fail (strchr (variable, '=') == NULL);
+
+  unsetenv (variable);
+#else
+  int len;
+  gchar **e, **f;
+
+  g_return_if_fail (strchr (variable, '=') == NULL);
+
+  len = strlen (variable);
+  
+  /* Mess directly with the environ array.
+   * This seems to be the only portable way to do this.
+   *
+   * Note that we remove *all* environment entries for
+   * the variable name, not just the first.
+   */
+  e = f = environ;
+  while (*e != NULL) 
+    {
+      if (strncmp (*e, variable, len) != 0 || (*e)[len] != '=') 
+	{
+	  *f = *e;
+	  f++;
+	}
+      e++;
+    }
+  *f = NULL;
+#endif
+}
 
 G_LOCK_DEFINE_STATIC (g_utils_global);
 
@@ -983,7 +1067,7 @@ g_get_tmp_dir (void)
   return g_tmp_dir;
 }
 
-G_LOCK_DEFINE (g_prgname);
+G_LOCK_DEFINE_STATIC (g_prgname);
 static gchar *g_prgname = NULL;
 
 gchar*
@@ -1007,7 +1091,7 @@ g_set_prgname (const gchar *prgname)
   G_UNLOCK (g_prgname);
 }
 
-G_LOCK_DEFINE (g_application_name);
+G_LOCK_DEFINE_STATIC (g_application_name);
 static gchar *g_application_name = NULL;
 
 /**
