@@ -327,6 +327,7 @@ main (int   argc,
   GPtrArray *gparray;
   GByteArray *gbarray;
   GString *string1, *string2;
+  const gchar *charset;
   GTree *tree;
   char chars[62];
   GRelation *relation;
@@ -401,16 +402,11 @@ main (int   argc,
   GError *error;
   char *name_used;
 #ifdef G_OS_WIN32
-  gchar *glib_dll = g_strdup_printf ("libglib-%d.%d-%d.dll",
-				     GLIB_MAJOR_VERSION,
-				     GLIB_MINOR_VERSION,
-				     GLIB_MICRO_VERSION - GLIB_BINARY_AGE);
+  /* Can't calculate GLib DLL name at runtime. */
+  gchar *glib_dll = "libglib-2.0-0.dll";
 #endif
 #ifdef G_WITH_CYGWIN
-  gchar *glib_dll = g_strdup_printf ("cygglib-%d.%d-%d.dll",
-				     GLIB_MAJOR_VERSION,
-				     GLIB_MINOR_VERSION,
-				     GLIB_MICRO_VERSION - GLIB_BINARY_AGE);
+  gchar *glib_dll = "cygglib-2.0-0.dll";
 #endif
 
   g_print ("TestGLib v%u.%u.%u (i:%u b:%u)\n",
@@ -429,14 +425,14 @@ main (int   argc,
   g_print ("tmp-dir: %s\n", g_get_tmp_dir ());
 
   /* type sizes */
-  g_print ("checking size of gint8: %d", (int)sizeof (gint8));
+  g_print ("checking size of gint8: %"    G_GSIZE_FORMAT, sizeof (gint8));
   TEST (NULL, sizeof (gint8) == 1);
-  g_print ("\nchecking size of gint16: %d", (int)sizeof (gint16));
+  g_print ("\nchecking size of gint16: %" G_GSIZE_FORMAT, sizeof (gint16));
   TEST (NULL, sizeof (gint16) == 2);
-  g_print ("\nchecking size of gint32: %d", (int)sizeof (gint32));
+  g_print ("\nchecking size of gint32: %" G_GSIZE_FORMAT, sizeof (gint32));
   TEST (NULL, sizeof (gint32) == 4);
-  g_print ("\nchecking size of gsize: %d", (int)sizeof (gsize));
-  g_print ("\nchecking size of gint64: %d", (int)sizeof (gint64));
+  g_print ("\nchecking size of gsize: %"  G_GSIZE_FORMAT, sizeof (gsize));
+  g_print ("\nchecking size of gint64: %" G_GSIZE_FORMAT, sizeof (gint64));
   TEST (NULL, sizeof (gint64) == 8);
   g_print ("\n");
 
@@ -1180,10 +1176,10 @@ main (int   argc,
 
   g_print ("ok\n");
 
-  if (g_get_charset (&string))
-    g_print ("current charset is UTF-8: %s\n", string);
+  if (g_get_charset (&charset))
+    g_print ("current charset is UTF-8: %s\n", charset);
   else
-    g_print ("current charset is not UTF-8: %s\n", string);
+    g_print ("current charset is not UTF-8: %s\n", charset);
 
 #ifdef G_PLATFORM_WIN32
   g_print ("current locale: %s\n", g_win32_getlocale ());
